<?php
/**
 * Copyright 2014-2018 Cherry Checkout SA
 *
 * NOTICE OF LICENSE
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  @author    Cherry Checkout SA <info@cherrycheckout.com>
 *  @copyright 2014-2018 Cherry Checkout SA
 *  @license   https://opensource.org/licenses/BSD-3-Clause  3-Clause BSD License (BSD-3-Clause)
 */

if (!function_exists('getallheaders')) {
    function getallheaders()
    {
        $headers = array ();
        foreach ($_SERVER as $name => $value) {
            if (Tools::substr($name, 0, 5) == 'HTTP_') {
                $replaceValue = ucwords(Tools::strtolower(str_replace('_', ' ', Tools::substr($name, 5))));
                $headers[str_replace(' ', '-', $replaceValue)] = $value;
            }
        }
        return $headers;
    }
}

require_once(dirname($_SERVER["SCRIPT_FILENAME"]).'/../../config/config.inc.php');

require_once('cherrycheckout.php');
require_once('class/CherryCheckoutAuth.php');

if (!defined('_PS_VERSION_')) {
    exit;
}

class CherryCheckoutPrestaWebservice
{
    private $idShop = 0;
    private $module;

    public function __construct($messageEncrypted, $timestamp)
    {
        $this->module = new CherryCheckout();

        try {
            $this->idShop = $this->getShopFromProvidedKeys($messageEncrypted, $timestamp);

            // No shop found, throw an error
            if (!$this->idShop) {
                throw new Exception('module is not active for this shop');
            }
        } catch (\Exception $ex) {
            throw $ex;
        }
    }

    private function getShopFromProvidedKeys($messageEncrypted, $timestamp)
    {
        if (Shop::isFeatureActive()) {
            // If multi shop is active, check them all
            $shops = Shop::getShops();
            foreach ($shops as $s) {
                Shop::setContext(Shop::CONTEXT_SHOP, $s['id_shop']);

                // Get keys for this shop
                $keys = $this->module->getApiKeys();
                $hash = CherryCheckoutAuth::hashKeys($keys['publicKey'], $keys['privateKey'], $timestamp);

                if ($messageEncrypted == $hash) {
                    return $s['id_shop'];
                }
            }
        } else {
            // Only one shop
            $keys = $this->module->getApiKeys();
            $hash = CherryCheckoutAuth::hashKeys($keys['publicKey'], $keys['privateKey'], $timestamp);

            if ($messageEncrypted == $hash) {
                $context = Context::getContext();
                return $context->shop->id;
            }
        }

        return false;
    }


    public function resetMe()
    {
        // For multi shop, set the good context
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_SHOP, $this->idShop);
        }

        try {
            $this->module->resetMe();

            return array(
                'result' => true,
                'message' => 'me reset'
            );
        } catch (Exception $ex) {
            return array(
                'result' => false,
                'message' => $ex->getMessage()
            );
        }
    }

    public function getLogs()
    {
        try {
            $logFile = $this->module->generateLogFile();

            return array(
                'result' => true,
                'message' => $logFile
            );
        } catch (\Exception $ex) {
            return array(
                'result' => false,
                'message' => $ex->getMessage()
            );
        }
    }
}

$response = null;

try {
    $headers = getallheaders();
    $authorization = null;
    $timestamp = false;

    // Get headers parameters
    foreach (getallheaders() as $name => $value) {
        if (Tools::strtolower($name) == 'authorization') {
            $authorization = explode(':', $value);
        } elseif (Tools::strtolower($name) == 'timestamp') {
            $timestamp = $value;
        }
    }

    // Check authorization parameters
    if (count($authorization) != 2 || !$timestamp) {
        throw new Exception('wrong authorization parameters');
    }

    $messageEncrypted = $authorization[1];
    $webservice = new CherryCheckoutPrestaWebservice($messageEncrypted, $timestamp);
} catch (\Exception $ex) {
    $response = array(
        'result' => false,
        'message' => $ex->getMessage()
    );
}


if (!Tools::getIsset('action')) {
    $response = array(
        'result' => false,
        'message' => 'action parameter is missing'
    );
} elseif (!!$webservice) {
    switch (Tools::getValue('action')) {
        case 'resetMe':
            $response = $webservice->resetMe();
            break;
        case 'getLogs':
            $response = $webservice->getLogs();
            break;
        default:
            $response = array(
                'result' => false,
                'message' => 'action parameter is wrong'
            );
            break;
    }
}

if ($response) {
    echo json_encode($response);
}
